@@ -63,8 +63,9 @@ GST_DEBUG_CATEGORY_STATIC (gst_dtls_srtp_enc_debug);
 
 #define gst_dtls_srtp_enc_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstDtlsSrtpEnc, gst_dtls_srtp_enc,
-    GST_TYPE_DTLS_SRTP_BIN, GST_DEBUG_CATEGORY_INIT (gst_dtls_srtp_enc_debug,
-        "dtlssrtpenc", 0, "DTLS Decoder"));
+    GST_TYPE_DTLS_SRTP_BIN,
+    GST_DEBUG_CATEGORY_INIT (gst_dtls_srtp_enc_debug,
+        "dtlssrtpenc", 0, "DTLS-SRTP Encoder"));
 GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (dtlssrtpenc, "dtlssrtpenc",
     GST_RANK_NONE, GST_TYPE_DTLS_SRTP_ENC, dtls_element_init (plugin));
 
@@ -186,7 +187,7 @@ gst_dtls_srtp_enc_init (GstDtlsSrtpEnc * self)
 {
   GstElementClass *klass = GST_ELEMENT_GET_CLASS (GST_ELEMENT (self));
   static GEnumClass *cipher_enum_class, *auth_enum_class;
-  gboolean ret;
+  gboolean ret GST_UNUSED_CHECKS;
 
 /*
                  +--------------------+    +--------------+     +-----------------+
@@ -395,6 +396,9 @@ gst_dtls_srtp_enc_request_new_pad (GstElement * element,
   g_return_val_if_fail (templ->direction == GST_PAD_SINK, NULL);
   g_return_val_if_fail (self->srtp_enc, NULL);
 
+  if (name == NULL)
+    return NULL;
+
   if (templ == gst_element_class_get_pad_template (klass, "rtp_sink_%d")) {
     gchar *clocksync_name;
     GstElement *clocksync;
@@ -500,7 +504,7 @@ gst_dtls_srtp_enc_remove_dtls_element (GstDtlsSrtpBin * bin)
 {
   GstDtlsSrtpEnc *self = GST_DTLS_SRTP_ENC (bin);
   GstPad *dtls_sink_pad, *peer_pad;
-  gulong id;
+  gulong id GST_UNUSED_CHECKS;
   guint rtp_cipher = 1, rtcp_cipher = 1, rtp_auth = 1, rtcp_auth = 1;
 
   if (!bin->dtls_element) {
