@@ -202,8 +202,8 @@ gst_va_base_dec_src_query (GstVideoDecoder * decoder, GstQuery * query)
         ret = TRUE;
         break;
       }
-      /* else jump to default */
     }
+      /* FALLTHROUGH */
     default:
       ret = GST_VIDEO_DECODER_CLASS (GST_VA_BASE_DEC_GET_PARENT_CLASS
           (decoder))->src_query (decoder, query);
@@ -227,7 +227,7 @@ _create_allocator (GstVaBaseDec * base, GstCaps * caps)
 {
   GstAllocator *allocator = NULL;
 
-  if (gst_caps_is_dmabuf (caps))
+  if (gst_video_is_dma_drm_caps (caps))
     allocator = gst_va_dmabuf_allocator_new (base->display);
   else {
     GArray *surface_formats =
@@ -250,6 +250,15 @@ _need_video_crop (GstVaBaseDec * base)
   return FALSE;
 }
 
+static GstVaFeature
+_use_derived (GstVaBaseDec * base)
+{
+  /* i965 driver has a very poor performance reading derived images */
+  if (GST_VA_DISPLAY_IS_IMPLEMENTATION (base->display, INTEL_I965))
+    return GST_VA_FEATURE_DISABLED;
+  return GST_VA_FEATURE_AUTO;
+}
+
 /* This path for pool setting is a little complicated but not commonly
    used. We deliberately separate it from the main path of pool setting. */
 static gboolean
@@ -257,7 +266,7 @@ _decide_allocation_for_video_crop (GstVideoDecoder * decoder,
     GstQuery * query, GstCaps * caps)
 {
   GstAllocator *allocator = NULL, *other_allocator = NULL;
-  GstAllocationParams other_params, params;
+  GstAllocationParams other_params = { 0, }, params = { 0, };
   gboolean update_pool = FALSE, update_allocator = FALSE;
   GstBufferPool *pool = NULL, *other_pool = NULL;
   guint size = 0, min, max, usage_hint;
@@ -350,7 +359,7 @@ _decide_allocation_for_video_crop (GstVideoDecoder * decoder,
         GST_BUFFER_POOL_OPTION_VIDEO_META);
 
     gst_buffer_pool_config_set_va_allocation_params (other_config,
-        VA_SURFACE_ATTRIB_USAGE_HINT_GENERIC, GST_VA_FEATURE_AUTO);
+        VA_SURFACE_ATTRIB_USAGE_HINT_GENERIC, _use_derived (base));
 
     if (!gst_buffer_pool_set_config (other_pool, other_config))
       goto cleanup;
@@ -363,7 +372,7 @@ _decide_allocation_for_video_crop (GstVideoDecoder * decoder,
 
   va_caps = gst_caps_copy (caps);
   gst_caps_set_features_simple (va_caps,
-      gst_caps_features_from_string (GST_CAPS_FEATURE_MEMORY_VA));
+      gst_caps_features_new_single_static_str (GST_CAPS_FEATURE_MEMORY_VA));
 
   if (!(allocator = _create_allocator (base, va_caps)))
     goto cleanup;
@@ -386,7 +395,7 @@ _decide_allocation_for_video_crop (GstVideoDecoder * decoder,
         VAEntrypointVLD, GST_PAD_SRC, gst_video_is_dma_drm_caps (caps));
 
     gst_buffer_pool_config_set_va_allocation_params (config,
-        usage_hint, GST_VA_FEATURE_AUTO);
+        usage_hint, _use_derived (base));
 
     if (!gst_buffer_pool_set_config (pool, config))
       goto cleanup;
@@ -466,7 +475,7 @@ static gboolean
 gst_va_base_dec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
 {
   GstAllocator *allocator = NULL, *other_allocator = NULL;
-  GstAllocationParams other_params, params;
+  GstAllocationParams other_params = { 0, }, params = { 0, };
   GstBufferPool *pool = NULL, *other_pool = NULL;
   GstCaps *caps = NULL;
   GstVaBaseDec *base = GST_VA_BASE_DEC (decoder);
@@ -475,7 +484,13 @@ gst_va_base_dec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
   gboolean has_videometa, has_video_crop_meta;
   gboolean dont_use_other_pool = FALSE, ret = FALSE;
 
-  g_assert (base->min_buffers > 0);
+  if (base->min_buffers == 0) {
+    GST_DEBUG_OBJECT (base,
+        "Don't know minimum number of required buffers yet");
+    return
+        GST_VIDEO_DECODER_CLASS (GST_VA_BASE_DEC_GET_PARENT_CLASS
+        (decoder))->decide_allocation (decoder, query);
+  }
 
   gst_query_parse_allocation (query, &caps, NULL);
 
@@ -487,6 +502,12 @@ gst_va_base_dec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
   has_video_crop_meta = has_videometa && gst_query_find_allocation_meta (query,
       GST_VIDEO_CROP_META_API_TYPE, NULL);
 
+  if (gst_video_is_dma_drm_caps (caps) && !has_videometa) {
+    GST_ERROR_OBJECT (base,
+        "DMABuf caps negotiated without the mandatory support of VideoMeta ");
+    return FALSE;
+  }
+
   /* 1. The output picture locates in the middle of the decoded buffer,
      but the downstream element does not support VideoCropMeta, we
      definitely need a copy.
@@ -568,7 +589,7 @@ gst_va_base_dec_decide_allocation (GstVideoDecoder * decoder, GstQuery * query)
         VAEntrypointVLD, GST_PAD_SRC, gst_video_is_dma_drm_caps (caps));
 
     gst_buffer_pool_config_set_va_allocation_params (config,
-        usage_hint, GST_VA_FEATURE_AUTO);
+        usage_hint, _use_derived (base));
 
     if (!gst_buffer_pool_set_config (pool, config))
       goto cleanup;
@@ -643,8 +664,7 @@ gst_va_base_dec_set_context (GstElement * element, GstContext * context)
   if (!ret
       || (old_display && new_display && old_display != new_display
           && base->decoder)) {
-    GST_ELEMENT_WARNING (base, RESOURCE, BUSY,
-        ("Can't replace VA display while operating"), (NULL));
+    GST_WARNING_OBJECT (base, "Can't replace VA display while operating");
   }
 
   gst_clear_object (&old_display);
@@ -659,27 +679,39 @@ gst_va_base_dec_negotiate (GstVideoDecoder * decoder)
 {
   GstVaBaseDec *base = GST_VA_BASE_DEC (decoder);
 
-  /* Ignore downstream renegotiation request. */
-  if (!base->need_negotiation)
-    return TRUE;
+  /* Do not (re-)open the decoder in case the input state hasn't changed. */
+  if (!base->need_negotiation) {
+    GST_DEBUG_OBJECT (decoder,
+        "Input state hasn't changed, no need to (re-)open the decoder");
+    goto done;
+  }
 
   base->need_negotiation = FALSE;
 
   if (!gst_va_decoder_config_is_equal (base->decoder, base->profile,
           base->rt_format, base->width, base->height)) {
     if (gst_va_decoder_is_open (base->decoder) &&
-        !gst_va_decoder_close (base->decoder))
+        !gst_va_decoder_close (base->decoder)) {
+      GST_WARNING_OBJECT (decoder, "Failed to close decoder");
       return FALSE;
-    if (!gst_va_decoder_open (base->decoder, base->profile, base->rt_format))
+    }
+    if (!gst_va_decoder_open (base->decoder, base->profile, base->rt_format)) {
+      GST_WARNING_OBJECT (decoder, "Failed to open decoder");
       return FALSE;
+    }
     if (!gst_va_decoder_set_frame_size (base->decoder, base->width,
-            base->height))
+            base->height)) {
+      GST_WARNING_OBJECT (decoder, "Failed to set frame size");
       return FALSE;
+    }
   }
 
-  if (!gst_va_base_dec_set_output_state (base))
+  if (!gst_va_base_dec_set_output_state (base)) {
+    GST_WARNING_OBJECT (decoder, "Failed to set output state");
     return FALSE;
+  }
 
+done:
   return GST_VIDEO_DECODER_CLASS (GST_VA_BASE_DEC_GET_PARENT_CLASS (decoder))
       ->negotiate (decoder);
 }
@@ -744,168 +776,103 @@ gst_va_base_dec_class_init (GstVaBaseDecClass * klass, GstVaCodecs codec,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
-/* XXX: if chroma has not an available format, the first format is
- * returned, relying on an hypothetical internal CSC */
+static inline GstVideoFormat
+_get_video_format_from_value (const GValue * format, gboolean drm_format,
+    guint64 * modifier)
+{
+  guint32 fourcc;
+  const char *fmt;
+
+  g_assert (G_VALUE_HOLDS_STRING (format));
+
+  fmt = g_value_get_string (format);
+
+  if (drm_format) {
+    fourcc = gst_video_dma_drm_fourcc_from_string (fmt, modifier);
+    return gst_va_video_format_from_drm_fourcc (fourcc);
+  }
+
+  return gst_video_format_from_string (fmt);
+}
+
 static GstVideoFormat
 _find_video_format_from_chroma (const GValue * formats, guint chroma_type,
     gboolean drm_format, guint64 * modifier)
 {
-  GstVideoFormat fmt;
-  guint32 fourcc;
+  GstVideoFormat fmt = GST_VIDEO_FORMAT_UNKNOWN;
   guint i, num_values;
 
   if (!formats)
     return GST_VIDEO_FORMAT_UNKNOWN;
 
   if (G_VALUE_HOLDS_STRING (formats)) {
-    if (drm_format) {
-      fourcc = gst_video_dma_drm_fourcc_from_string
-          (g_value_get_string (formats), modifier);
-      return gst_va_video_format_from_drm_fourcc (fourcc);
-    } else {
-      return gst_video_format_from_string (g_value_get_string (formats));
-    }
+    fmt = _get_video_format_from_value (formats, drm_format, modifier);
+    if (gst_va_chroma_from_video_format (fmt) == chroma_type)
+      return fmt;
   } else if (GST_VALUE_HOLDS_LIST (formats)) {
-    GValue *val, *first_val = NULL;
+    const GValue *format;
 
     num_values = gst_value_list_get_size (formats);
     for (i = 0; i < num_values; i++) {
-      val = (GValue *) gst_value_list_get_value (formats, i);
-      if (!val)
-        continue;
-      if (!first_val)
-        first_val = val;
-
-      if (drm_format) {
-        fourcc = gst_video_dma_drm_fourcc_from_string (g_value_get_string (val),
-            modifier);
-        fmt = gst_va_video_format_from_drm_fourcc (fourcc);
-      } else {
-        fmt = gst_video_format_from_string (g_value_get_string (val));
-      }
+      format = gst_value_list_get_value (formats, i);
+      fmt = _get_video_format_from_value (format, drm_format, modifier);
 
       if (gst_va_chroma_from_video_format (fmt) == chroma_type)
         return fmt;
     }
-
-    if (first_val) {
-      if (drm_format) {
-        fourcc = gst_video_dma_drm_fourcc_from_string (g_value_get_string
-            (first_val), modifier);
-        return gst_va_video_format_from_drm_fourcc (fourcc);
-      } else {
-        return gst_video_format_from_string (g_value_get_string (first_val));
-      }
-    }
   }
 
   return GST_VIDEO_FORMAT_UNKNOWN;
 }
 
 static GstVideoFormat
-_caps_video_format_from_chroma (GstCaps * caps, GstCapsFeatures * features,
-    guint chroma_type, guint64 * ret_modifier)
+_caps_video_format_from_chroma (GstCaps * caps, guint chroma_type)
 {
   guint i, num_structures;
-  gboolean drm_format;
   GstCapsFeatures *feats;
   GstStructure *structure;
   const GValue *format;
-  GstVideoFormat fmt, ret_fmt = GST_VIDEO_FORMAT_UNKNOWN;
-  guint64 modifier;
+  GstVideoFormat fmt;
 
   num_structures = gst_caps_get_size (caps);
   for (i = 0; i < num_structures; i++) {
     feats = gst_caps_get_features (caps, i);
-    if (!gst_caps_features_is_equal (feats, features))
+    if (!gst_caps_features_is_equal (feats,
+            GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY))
       continue;
     structure = gst_caps_get_structure (caps, i);
+    format = gst_structure_get_value (structure, "format");
 
-    if (gst_caps_features_contains (feats, GST_CAPS_FEATURE_MEMORY_DMABUF)) {
-      format = gst_structure_get_value (structure, "drm-format");
-      drm_format = TRUE;
-    } else {
-      format = gst_structure_get_value (structure, "format");
-      drm_format = FALSE;
-    }
-
-    fmt = _find_video_format_from_chroma (format, chroma_type, drm_format,
-        &modifier);
+    fmt = _find_video_format_from_chroma (format, chroma_type, FALSE, NULL);
     if (fmt == GST_VIDEO_FORMAT_UNKNOWN)
       continue;
-
-    if (ret_fmt == GST_VIDEO_FORMAT_UNKNOWN) {
-      ret_fmt = fmt;
-      if (ret_modifier)
-        *ret_modifier = modifier;
-    }
-
-    if (gst_va_chroma_from_video_format (fmt) == chroma_type) {
-      ret_fmt = fmt;
-      if (ret_modifier)
-        *ret_modifier = modifier;
-      break;
-    }
+    if (gst_va_chroma_from_video_format (fmt) == chroma_type)
+      return fmt;
   }
 
-  return ret_fmt;
+  return GST_VIDEO_FORMAT_UNKNOWN;
 }
 
-static GstVideoFormat
-_default_video_format_from_chroma (GstVaBaseDec * base,
-    GstCaps * preferred_caps, GstCapsFeatures * features, guint chroma_type,
-    guint64 * modifier)
-{
-  GstCaps *tmpl_caps;
-  GstVideoFormat ret = GST_VIDEO_FORMAT_UNKNOWN;
-
-  tmpl_caps = gst_pad_get_pad_template_caps (GST_VIDEO_DECODER_SRC_PAD (base));
-
-  /* Make the preferred caps in the order of our template */
-  if (preferred_caps) {
-    GstCaps *tmp;
-    g_assert (!gst_caps_is_empty (preferred_caps));
-
-    tmp = tmpl_caps;
-    tmpl_caps = gst_caps_intersect_full (tmp, preferred_caps,
-        GST_CAPS_INTERSECT_FIRST);
-    gst_caps_unref (tmp);
-  }
-
-  ret = _caps_video_format_from_chroma (tmpl_caps, features, chroma_type,
-      modifier);
-
-  gst_caps_unref (tmpl_caps);
-
-  return ret;
-}
-
-/* Check whether the downstream supports VideoMeta; if not, we need to
- * fallback to the system memory. */
-static gboolean
-_downstream_has_video_meta (GstVaBaseDec * base, GstCaps * caps)
-{
-  GstQuery *query;
-  gboolean ret = FALSE;
-
-  query = gst_query_new_allocation (caps, FALSE);
-  if (gst_pad_peer_query (GST_VIDEO_DECODER_SRC_PAD (base), query))
-    ret = gst_query_find_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL);
-  gst_query_unref (query);
-
-  return ret;
-}
+/* ordered list of capsfeature preference */
+enum
+{ VA, DMABUF, SYSMEM };
 
 void
 gst_va_base_dec_get_preferred_format_and_caps_features (GstVaBaseDec * base,
     GstVideoFormat * format, GstCapsFeatures ** capsfeatures,
     guint64 * modifier)
 {
-  GstCaps *peer_caps = NULL, *preferred_caps = NULL;
+  GstCaps *peer_caps, *allowed_caps;
   GstCapsFeatures *features;
-  GstStructure *structure;
-  guint num_structures, i;
+  guint num_structures, i, j;
   gboolean is_any;
+  GstIdStr sysmem = GST_ID_STR_INIT, dmabuf = GST_ID_STR_INIT, va =
+      GST_ID_STR_INIT;
+  const GstIdStr *feats[] = { &va, &dmabuf, &sysmem };
+
+  gst_id_str_set_static_str (&sysmem, GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
+  gst_id_str_set_static_str (&dmabuf, GST_CAPS_FEATURE_MEMORY_DMABUF);
+  gst_id_str_set_static_str (&va, GST_CAPS_FEATURE_MEMORY_VA);
 
   g_return_if_fail (base);
 
@@ -917,70 +884,70 @@ gst_va_base_dec_get_preferred_format_and_caps_features (GstVaBaseDec * base,
     gst_clear_caps (&peer_caps);
   }
 
-  peer_caps = gst_pad_get_allowed_caps (GST_VIDEO_DECODER_SRC_PAD (base));
-  GST_DEBUG_OBJECT (base, "Allowed caps %" GST_PTR_FORMAT, peer_caps);
+  allowed_caps = gst_pad_get_allowed_caps (GST_VIDEO_DECODER_SRC_PAD (base));
+  GST_DEBUG_OBJECT (base, "Allowed caps %" GST_PTR_FORMAT, allowed_caps);
 
-  /* prefer memory:VASurface over other caps features */
-  num_structures = gst_caps_get_size (peer_caps);
-  for (i = 0; i < num_structures; i++) {
-    features = gst_caps_get_features (peer_caps, i);
-    structure = gst_caps_get_structure (peer_caps, i);
+  /* if peer caps is any, returns format according memory to system caps in
+   * allowed caps
+   *
+   * if downstream doesn't support system memory negotiation will fail later.
+   */
+  if (is_any) {
+    GstVideoFormat fmt =
+        _caps_video_format_from_chroma (allowed_caps, base->rt_format);
+    features = GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY;
 
-    if (gst_caps_features_is_any (features))
-      continue;
-
-    if (gst_caps_features_contains (features, GST_CAPS_FEATURE_MEMORY_VA)) {
-      preferred_caps = gst_caps_new_full (gst_structure_copy (structure), NULL);
-      gst_caps_set_features_simple (preferred_caps,
-          gst_caps_features_copy (features));
-      break;
-    }
-  }
-
-  if (!preferred_caps)
-    preferred_caps = gst_caps_copy (peer_caps);
-
-  if (gst_caps_is_empty (preferred_caps)) {
-    if (capsfeatures)
-      *capsfeatures = NULL;     /* system memory */
-    if (format) {
-      *format = _default_video_format_from_chroma (base, NULL,
-          GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY, base->rt_format, NULL);
-    }
+    if (format)
+      *format = fmt;
+    if (capsfeatures && fmt != GST_VIDEO_FORMAT_UNKNOWN)
+      *capsfeatures = gst_caps_features_copy (features);
     goto bail;
   }
 
-  /* Use the first structure/feature is caps because is the
-   * "preferred" one */
-  features = gst_caps_get_features (preferred_caps, 0);
-  if (!features) {
-    features = GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY;
-  } else if (is_any
-      && !gst_caps_features_is_equal (features,
-          GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY)
-      && !_downstream_has_video_meta (base, preferred_caps)) {
-    GST_INFO_OBJECT (base, "Downstream reports ANY caps but without"
-        " VideoMeta support; fallback to system memory.");
+  /* iterate allowed caps to find the first "capable" capability according our
+   * ordered list of preferred caps features */
+  num_structures = gst_caps_get_size (allowed_caps);
+  for (i = 0; i < G_N_ELEMENTS (feats); i++) {
+    for (j = 0; j < num_structures; j++) {
+      GstStructure *structure;
+      const GValue *formats;
+      GstVideoFormat fmt;
+      guint64 mod = 0;
 
-    features = GST_CAPS_FEATURES_MEMORY_SYSTEM_MEMORY;
-    gst_clear_caps (&preferred_caps);
-    preferred_caps = gst_caps_copy (peer_caps);
+      features = gst_caps_get_features (allowed_caps, j);
+      if (!gst_caps_features_contains_id_str (features, feats[i]))
+        continue;
+
+      structure = gst_caps_get_structure (allowed_caps, j);
+      if (i == DMABUF)
+        formats = gst_structure_get_value (structure, "drm-format");
+      else
+        formats = gst_structure_get_value (structure, "format");
+
+      fmt = _find_video_format_from_chroma (formats, base->rt_format,
+          i == DMABUF, &mod);
+
+      /* if doesn't found a proper format let's try other structure */
+      if (fmt == GST_VIDEO_FORMAT_UNKNOWN)
+        continue;
+
+      if (format)
+        *format = fmt;
+      if (i == DMABUF && modifier)
+        *modifier = mod;
+      if (capsfeatures)
+        *capsfeatures = gst_caps_features_new_id_str (feats[i], NULL);
+
+      goto bail;
+    }
   }
 
-
-  if (capsfeatures)
-    *capsfeatures = gst_caps_features_copy (features);
-
-  /* Use the format from chroma and available format for selected
-   * capsfeature */
-  if (format) {
-    *format = _default_video_format_from_chroma (base, preferred_caps,
-        features, base->rt_format, modifier);
-  }
+  /* no matching format, let's fail */
+  if (i == G_N_ELEMENTS (feats))
+    *format = GST_VIDEO_FORMAT_UNKNOWN;
 
 bail:
-  gst_clear_caps (&preferred_caps);
-  gst_clear_caps (&peer_caps);
+  gst_caps_unref (allowed_caps);
 }
 
 static gboolean
